# shim file for testing purpose,
# _find_caller_version() should be called from a package

from neuro_logging.trace import _find_caller_version


def _get_test_version() -> str:
    return _find_caller_version(1)



from __future__ import annotations

import logging
import os
from dataclasses import dataclass


@dataclass(frozen=True)
class LoggingConfig:
    log_level: int = logging.INFO
    log_health_check: bool = False


@dataclass(frozen=True)
class SentryConfig:
    dsn: str | None = None
    cluster_name: str | None = None
    app_name: str | None = None
    sample_rate: float = 0.1


def _to_bool(value: str) -> bool:
    return value.lower() in ("true", "1", "yes", "y")


class EnvironConfigFactory:
    def __init__(self, environ: dict[str, str] | None = None) -> None:
        self._environ = environ or os.environ

    def create_logging(self) -> LoggingConfig:
        return LoggingConfig(
            log_level=logging.getLevelName(
                self._environ.get(
                    "LOG_LEVEL",
                    logging.getLevelName(LoggingConfig.log_level),
                ).upper()
            ),
            log_health_check=_to_bool(self._environ.get("LOG_HEALTH_CHECK", "0")),
        )

    def create_sentry(self) -> SentryConfig:
        return SentryConfig(
            dsn=self._environ.get("SENTRY_DSN"),
            cluster_name=self._environ.get(
                "SENTRY_CLUSTER_NAME", SentryConfig.cluster_name
            ),
            app_name=self._environ.get("SENTRY_APP_NAME", SentryConfig.app_name),
            sample_rate=float(
                self._environ.get("SENTRY_SAMPLE_RATE", SentryConfig.sample_rate)
            ),
        )



import logging
import os
import re
from typing import Any

import pytest

from neuro_logging import AllowLessThanFilter, init_logging


@pytest.fixture(autouse=True)
def set_log_level() -> None:
    os.environ["LOG_LEVEL"] = "NOTSET"


def _log_all_messages() -> None:
    logging.debug("DebugMessage")
    logging.info("InfoMessage")
    logging.warning("WarningMessage")
    logging.error("ErrorMessage")
    logging.critical("CriticalMessage")


def test_default_config_format(capsys: Any) -> None:
    init_logging()
    logging.debug("DebugMessage")
    captured = capsys.readouterr()
    assert re.match(
        r"\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2},\d+ - root - DEBUG - DebugMessage",
        captured.out,
    )


def test_default_config_output(capsys: Any) -> None:
    init_logging()
    _log_all_messages()
    captured = capsys.readouterr()
    assert "DebugMessage" in captured.out
    assert "InfoMessage" in captured.out
    assert "WarningMessage" in captured.out
    assert "ErrorMessage" in captured.err
    assert "CriticalMessage" in captured.err


def test_health_checks_filtered(capsys: Any) -> None:
    init_logging()
    logging.getLogger("aiohttp.access").info("InfoMessage")
    logging.getLogger("aiohttp.access").info("GET /api/v1/ping")
    captured = capsys.readouterr()
    assert "InfoMessage" in captured.out
    assert "/api/v1/ping" not in captured.out


def test_health_checks_filtered__error(capsys: Any) -> None:
    init_logging()
    logging.getLogger("aiohttp.access").error("GET /api/v1/ping")
    captured = capsys.readouterr()
    assert "/api/v1/ping" in captured.err


def test_health_checks_filtered__custom_url_path(capsys: Any) -> None:
    init_logging(health_check_url_path="/health")
    logging.getLogger("aiohttp.access").info("GET /health")
    captured = capsys.readouterr()
    assert not captured.out


def test_allow_less_filter_usage() -> None:
    filter = AllowLessThanFilter(logging.INFO)
    record_info = logging.LogRecord("some", logging.INFO, "some", 12, "text", (), None)
    record_debug = logging.LogRecord(
        "some", logging.DEBUG, "some", 12, "text", (), None
    )
    assert filter.filter(record_info) is False
    assert filter.filter(record_debug) is True


def test_allow_less_filter_text_level_names() -> None:
    filter = AllowLessThanFilter("INFO")
    assert filter.level == logging.INFO

    with pytest.raises(ValueError):
        AllowLessThanFilter("unknown-level")


def test_existing_loggers_continue_work(capsys: Any) -> None:
    existing = logging.getLogger("existing")
    init_logging()

    existing.info("InfoMessage")
    existing.error("ErrorMessage")
    captured = capsys.readouterr()
    assert "InfoMessage" in captured.out
    assert "ErrorMessage" in captured.err
